<?php
namespace PME\Modules\Planejamento\Prato;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;

class PratoController extends \PME\Services\TeknisaCrud {
    protected $crudService;
	protected $parameterBag;
	protected $connection;
	protected $environment;
	protected $dataSourceManager;
	protected $utilities;
	protected $entityManager;
	protected $user;
    protected $newCode;
    protected $fileUploader;
    protected $imgPathURL;

    public function __construct($crudService, $parameterBag, $connection, $environment, $dataSourceManager, $utilities, $entityManager, $newCode, $fileUploader, $imgPathURL) {
        parent::__construct($crudService, $parameterBag);
        $this->connection        = $connection;
        $this->environment       = $environment;
		$this->dataSourceManager = $dataSourceManager;
		$this->utilities         = $utilities;
		$this->entityManager     = $entityManager;
	    $this->newCode = $newCode;
	    $this->fileUploader = $fileUploader;
	    $this->imgPathURL = $imgPathURL;
	}
	
	private function callSave($dataSourceName, $rowsToSave){
	    $dataSet = new DataSet($dataSourceName, $rowsToSave);
	    $request = new DTO\Request\DataSet($dataSet, "", "", "");  
	    $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
	    $this->crudService->save($dataSourceName, $rowsToSave);
	}
    
    public function returnParamsAddPlate(DTO\Request\Row $request, DTO\Response $response){
    	$row = $request->getRow();
    	$this->validationDependingGroup($request, $response);
        
        $filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('paramgeral', array());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $response->addDataSet($dataSet);
    }

	public function searchNewCode(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$nrorg = $this->utilities->getNrOrg();
		$pratoPai = $parameters['pratoPai'];
		$filAux = $parameters['filAux'];
		$levelPlate = $parameters['levelPlate'];
		$filial = $parameters['unity']['CDFILIAL'];

		$lastChildren = null;
		$maxCodePlate = 0;
		$maskPlate = $parameters['mask'];
		$maskNumbersLength = strlen(str_replace(".", "", $maskPlate));
		$padRightParentPlate = str_pad($pratoPai, $maskNumbersLength, "X", STR_PAD_RIGHT);
		$children = $this->returnChildren($pratoPai, $filAux, $filial, $nrorg);

		if (count($children) == 0 && $pratoPai === 0) $children = $this->returnLevelComplete($nrorg); //caso não tenha nenhum registro selecionado
		sort($children);
		
		
		if ($children) {
			$lastPosition = end($children);
			$lastChildren = $lastPosition['CDPRATO'];
		}
		
	
		if ($lastChildren === null) {
			$formatCode = $this->getFormatPlate($padRightParentPlate, $maskPlate);
			$matches = array();
			preg_match("/X+/", $formatCode, $matches);
		
			$sizeToComplete = strlen($matches[0]);
			$maxCodePlate = str_pad($pratoPai, $sizeToComplete+strlen($pratoPai), "0", STR_PAD_RIGHT);
		} else {
			$formatCodeLastPlate = $this->getFormatPlate($lastChildren, $maskPlate);
			$arrayCode = explode(".", $formatCodeLastPlate);
			$lastPart = $arrayCode[count($arrayCode)-1];
			if(preg_match("/^9+$/", $lastPart) == 1){
                $maxCodePlate = $lastChildren; 
            } else {
                $maxCodePlate = $lastChildren + 1;
            }
		}
		$response->addDataSet(new DataSet ('CODIGO', array($maxCodePlate)));
	}

	public function returnChildren($pratoPai, $filAux, $filial, $nrorg) {
		$sql = "SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL, NMPRATO, CDNVPRATO FROM PRATO WHERE CDPRATONIVANTE = :P_CDPRATONIVANTE AND NRORG = :P_NRORG AND CDFILAUXPRAT IN (:P_CDFILAUXPRAT, :P_CDFILIAL)";

		$params = array(
			"P_CDPRATONIVANTE" => $pratoPai,
			"P_NRORG"          => $nrorg,
			"P_CDFILAUXPRAT"   => $filAux,
			"P_CDFILIAL"       => $filial
		);
		return $this->entityManager->getConnection()->fetchAll($sql, $params);
	}
	
	public function returnLevelComplete($nrorg) {
		$sql = "SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL, NMPRATO, CDNVPRATO FROM PRATO WHERE CDNVPRATO = '1' AND NRORG = :P_NRORG ORDER BY CDPRATO";

		$params = array(
			
			"P_NRORG" => $nrorg
		);
		return $this->entityManager->getConnection()->fetchAll($sql, $params);
	}

	public function getFormatPlate($code, $mask){
        $regex = '';
        $toReplace = '';
        preg_match_all("/[^\.]+/", $mask, $results);
        $results = $results[0];
        for($i = 0; $i < sizeof($results); $i++){
            $sizeMatch = strlen($results[$i]);
            $regex .= $i === 0 ? "(.{{$sizeMatch}})" : "(.{0,$sizeMatch})";
            $toReplace .= $i !== 0 ? '.' : $toReplace;
            $toReplace .= "$".($i+1);
        }
        $formatCode = preg_replace("/".$regex."/", $toReplace, $code);
        $cleanCode = preg_replace("/(\.+)$/","",$formatCode);
	        
        return $cleanCode;
    }

	public function returnPlate(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();

		$plate = $parameters['CDPRATO'];
		$plateLocalIqualPadr = $parameters['paramPL'];
		$filAux = $parameters['filAux'];
		
		$nrorg = $this->utilities->getNrOrg();

		
		if ($plateLocalIqualPadr === 'S') {
			$sql = "SELECT * FROM PRATO WHERE CDPRATO = :CDPRATO AND NRORG = :NRORG ORDER BY CDPRATO DESC";
			$params = array(
				"CDPRATO"      => $plate,
				"NRORG"        => $nrorg
			);
		} else {
			$sql = "SELECT * FROM PRATO WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND NRORG = :NRORG ORDER BY CDPRATO DESC";
			$params = array(
				"CDFILAUXPRAT" => $filAux,
				"CDPRATO"      => $plate,
				"NRORG"        => $nrorg
			);
		}
		
		$result = $this->entityManager->getConnection()->fetchAll($sql, $params);

		if ($result) $response->addDataSet(new DataSet('PRATO', array('CDPRATO' => $result[0]['CDPRATO'], 'CDFILAUXPRAT' => $result[0]['CDFILAUXPRAT'])));
		else $response->addDataSet(new DataSet('PRATO', array('CDPRATO' => false)));
	}

	public function updatePackaging(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$rows = $parameters['update'];
		$nrorg = $this->utilities->getNrOrg();
		$this->connection->beginTransaction();
		$sql = "UPDATE PRATOCONTAINER SET IDPRATCONTPRI = :IDPRATCONTPRI WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND CDCONTAINER = :CDCONTAINER AND QTPRATCONTAINER = :QTPRATCONTAINER AND NRORG = :NRORG";

		$params = array(
			"CDFILAUXPRAT"    => $rows['CDFILAUXPRAT'],
			"CDPRATO"         => $rows['CDPRATO'],
			"CDCONTAINER"     => $rows['CDCONTAINER'],
			"QTPRATCONTAINER" => $rows['QTPRATCONTAINER'],
			"IDPRATCONTPRI"   => 'N',
			"NRORG"           => $nrorg
		);
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		$this->connection->commit();
	}
	
	public function savePlate(DTO\Request\Row $request, DTO\Response $response) {
		try {
			$parameters = $request->getParameters();
			$row = $parameters['rowForm'];
			$nrorg = $this->utilities->getNrOrg();

			if (isset($row["DSUNIDADE"]['SGUNIDADE']) == true) $dsunidade = $row["DSUNIDADE"]['SGUNIDADE'];
			else if (isset($row["DSUNIDADEDP"]) == true) $dsunidade = $row["DSUNIDADEDP"];
			else $dsunidade = $row["DSUNIDADE"];
		
			if (\Zeedhi\Framework\Util\Functions::arrayKeyExists("NMFORMAPREPARODP", $row) == true) $nmFormaPrep = $row["NMFORMAPREPARODP"];
			else $nmFormaPrep =  $row["NMFORMAPREPARO"];
			
			$sql = "INSERT INTO PRATO 
						(CDFILIAL, CDPRATO, NMPRATO, CDFILAUXPRAT, CDNVPRATO, 
						SGUNIDADE, VRARREDTRANS, QTTEMPOPRAT, IDFREQUENCIA, 
						IDATIVO, DTINATIVAUTO, NRFORMAPREPARO, NRORG) 
				 VALUES (:CDFILIAL, :CDPRATO, :NMPRATO, :CDFILAUXPRAT, :CDNVPRATO, 
						:SGUNIDADE, :VRARREDTRANS, :QTTEMPOPRAT, :IDFREQUENCIA, 
						:IDATIVO, :DTINATIVAUTO, :NRFORMAPREPARO, :NRORG)";
			
			$params = array(
				"CDFILIAL"       => $row['CDFILAUXPRAT'] === 'L' ? $row["CDFILIAL"] : '',
				"CDPRATO"        => $row["CDPRATO"],
				"NMPRATO"        => $row["NMPRATO"],
				"CDFILAUXPRAT"   => $row['CDFILAUXPRAT'] === 'L' ? $row['CDFILIAL'] : $row['CDFILAUXPRAT'],
				"CDNVPRATO"      => $row["CDNVPRATO"],
				"SGUNIDADE"      => $dsunidade,
				"VRARREDTRANS"   => $row["VRARREDTRANS"],
				"QTTEMPOPRAT"    => $row["QTTEMPOPRAT"],
				"IDFREQUENCIA"   => $row["IDFREQUENCIA"],
				"IDATIVO"        => $row["IDATIVO"],
				"DTINATIVAUTO"   => $row["DTINATIVAUTO"],
				"NRFORMAPREPARO" => $nmFormaPrep,
				"NRORG"          => $nrorg
			);

			$stmt = $this->connection->prepare($sql);
			$stmt->execute($params);
		} catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
	}
	
	public function updatePlate(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$rows = $parameters['editedRows'];
		$nrorg = $this->utilities->getNrOrg();
		$this->connection->beginTransaction();

		$sql = "UPDATE PRATO SET NMPRATO = :NMPRATO WHERE CDNVPRATO = :CDNVPRATO AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND NRORG = :NRORG";
		forEach($rows as $row) {
			$params = array(
				"CDPRATO"        => $row["CDPRATO"],
				"NMPRATO"        => $row["NMPRATO"],
				"CDFILAUXPRAT"   => $row["CDFILAUXPRAT"],
				"CDNVPRATO"      => $row["CDNVPRATO"],
				"NRORG"          => $nrorg
			);
			
			$stmt = $this->connection->prepare($sql);
			$stmt->execute($params);
		}
		
		$this->connection->commit();
	}
	
	static function cmp($a, $b) {
	    return strcmp($a["CDPRATO"], $b["CDPRATO"]);
	} 
	
	public function deletePlate(DTO\Request $request, DTO\Response $response) {
		try {
		  $this->connection->beginTransaction();
          $parameters = $request->getParameters();
	      $rows = $parameters['editedRows'];
	      usort($rows, array("self", "cmp"));
		  krsort($rows);
		  $return = true;
		  
	      $nrorg = $this->utilities->getNrOrg();
	      $query = "DELETE FROM PRATO WHERE CDNVPRATO = :CDNVPRATO AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND NRORG = :NRORG";
  
	      foreach($rows as $row){
	        $params = array(
				"CDPRATO"        => $row["CDPRATO"],
				// "CDFILAUXPRAT"   => $row["CDFILAUXPRAT"],
				// sem isso não exclui prato local 
				"CDFILAUXPRAT"   => ($row["CDFILAUXPRAT"] === 'L') ? $row["CDFILIAL"] : $row["CDFILAUXPRAT"],
				"CDNVPRATO"      => $row["CDNVPRATO"],
				"NRORG"          => $nrorg
			);
			// verifica se tem filho
			$chield = $this->returnChildren($row["CDPRATO"], $row["CDFILAUXPRAT"], $row["CDFILIAL"], $nrorg);
			if (count($chield) > 0) {
				$return = 'ITENS_POSSUIAM_FILHOS';
			} else {
		    	$this->connection->prepare($query)->execute($params);
			}
	      }
		  $this->connection->commit();
          $response->addDataSet(new DataSet("success", array($return)));
		} catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
	
	public function resultDadosPlate(DTO\Request\Row $request, DTO\Response $response) {
		// $parameters = $request->getParameters();
		// $rows = $parameters['currentRow'];
		$row = $request->getRow();
		$nrorg = $this->utilities->getNrOrg();
		
		if ($row['CDFILAUXPRAT'] == null) $row['CDFILAUXPRAT'] = $row['CDFILIAL'];
		
		$sql = "SELECT P.VRARREDTRANS, P.QTTEMPOPRAT, P.IDFREQUENCIA, 
					   P.DTINATIVAUTO, P.CDFILIAL, P.IDATIVO, P.CDNVPRATO, 
					   P.CDPRATONIVANTE, P.CDPRATO, P.CDFILAUXPRAT, 
					   P.NMPRATO, 
					   CASE P.CDFILAUXPRAT WHEN 'P' THEN 'P' ELSE 'L' END AS IDTIPO, 
					   P.SGUNIDADE, UN.DSUNIDADE, P.NRFORMAPREPARO, 
					   FP.NMFORMAPREPARO, P.NRORG 
				  FROM PRATO P 
				  LEFT JOIN UNIDMEDIDA UN ON UN.SGUNIDADE = P.SGUNIDADE 
				  LEFT JOIN FORMAPREPARO FP ON P.NRFORMAPREPARO = FP.NRFORMAPREPARO 
				 WHERE P.CDPRATO = :CDPRATO 
				   AND P.CDFILAUXPRAT = :CDFILAUXPRAT 
				   AND P.NRORG = :NRORG";
		
		$params = array(
			"CDPRATO"        => $row["CDPRATO"],
			"CDFILAUXPRAT"   => $row["CDFILAUXPRAT"],
			"NRORG"          => $nrorg
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}
	
	public function updateDadosPlate(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();
		$rows = $parameters['rowDadosPlate'];
		$nrorg = $this->utilities->getNrOrg();

		//if ($rows['CDFILAUXPRAT'] !== 'P') $rows['CDFILAUXPRAT'] = $rows['CDFILIAL'];
		if ($rows['NMFORMAPREPARODP'] == null) $nrFormaPrep = NULL;
		// else $nrFormaPrep = $rows['NMFORMAPREPARODP']['NRFORMAPREPARO'];
		else $nrFormaPrep = $rows['NMFORMAPREPARODP'];

		$sql = "UPDATE PRATO 
				   SET NMPRATO = :NMPRATO, SGUNIDADE = :SGUNIDADE, 
					   VRARREDTRANS = :VRARREDTRANS, 
					   QTTEMPOPRAT = :QTTEMPOPRAT, IDFREQUENCIA = :IDFREQUENCIA, 
					   NRFORMAPREPARO = :NRFORMAPREPARO, IDATIVO = :IDATIVO, DTINATIVAUTO = :DTINATIVAUTO 
				 WHERE CDPRATO = :CDPRATO 
				   AND CDFILAUXPRAT = :CDFILAUXPRAT AND NRORG = :NRORG";
		$params = array(
			"NMPRATO"        => $rows["NMPRATO"],
			//"SGUNIDADE"      => $rows["DSUNIDADEDP"]["SGUNIDADE"],
			"SGUNIDADE"      => $rows["DSUNIDADEDP"],
			"VRARREDTRANS"   => isset($rows["VRARREDTRANS"]) ? $rows["VRARREDTRANS"] : '',
			"QTTEMPOPRAT"    => isset($rows["QTTEMPOPRAT"]) ? $rows["QTTEMPOPRAT"] : '',
			"IDFREQUENCIA"   => $rows["IDFREQUENCIA"],
			"NRFORMAPREPARO" => $nrFormaPrep,
			"IDATIVO"        => $rows["IDATIVO"],
			"CDPRATO"        => $rows["CDPRATO"],
			"CDFILAUXPRAT"   => $rows["CDFILAUXPRAT_UPDATE"],
			"NRORG"          => $nrorg,
			"DTINATIVAUTO"   => $rows['DTINATIVAUTO']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
	}
	
	public function returnParamTco(DTO\Request\Row $request, DTO\Response $response) {
		$nrorg = $this->utilities->getNrOrg();
		$sql = "SELECT IDREVRECPADQREGIAO FROM PARAMTCO WHERE NRORG = :NRORG AND NRPARAMTCO = 1";
		$params = array(
			"NRORG" => $nrorg
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
    	if ($result == []){
    		$paramTCO = ['N'];
    	} else $paramTCO = $result;
		$response->addDataSet(new DataSet("paramTco", $paramTCO));
	}
	
	public function returnPercentSafra(DTO\Request\Row $request, DTO\Response $response) {
		$parameters = $request->getParameters();

		$sql = "SELECT NRCORSINSAFR FROM SINASAFRA WHERE VRLIMINFSAFR <  :P_PERCSAFRA AND VRLIMSUPSAFR >= :P_PERCSAFRA OR :P_PERCSAFRA =  0 AND VRLIMINFSAFR =  :P_PERCSAFRA";
		
		$params = array(
			"P_PERCSAFRA" => $parameters['percentualSafra']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}
	
	public function returnOperGroup(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();

		$sql = "SELECT CDGRUPOPER, CDOPERADOR FROM OPERGRUPOP WHERE CDOPERADOR = :CDOPERADOR";
		
		$params = array(
			"CDOPERADOR" => $row['CDOPERADOR']
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}
	
	public function saveOperator(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();
        $rowAuth = array(
            "AUTENTICACAO" => array(
                "ALIAS_NAME"  => "ALTRECEPADRAO",
                "COLUMN_NAME" => "CDGROPERREPA"
            ),
            "CDOPERADOR"  => $this->environment->getCdOperador()
        );
        $email = $row['DSEMAILOPER'] ?? null;
        $password = $row['CDSENHAOPERWEB'] ?? null;
        $rowAuth['EMAIL'] = $email;
        $rowAuth['SENHA'] = $password;
        $requestAuth = new DTO\Request\Row($rowAuth, null, null, null);
        $responseAuth = new DTO\Response();
        
        if($email && $password) $this->authOperator($requestAuth, $responseAuth);
        else $this->isValidAccess($requestAuth, $responseAuth);
        $responseError = $responseAuth->getError();
        
        if ($responseError != NULL) $msg = array('message' => $responseError->getMessage(), 'type' => 'error');
        else $msg = array('message' => null, 'type' => 'success');
        
        $dataSetsAuth = $responseAuth->getDataSets();
        $dataSetPermission = array_filter($dataSetsAuth, function($dataSet) {
            return $dataSet->getDataSourceName() == 'permission';
        })[0] ?? NULL;

        if($dataSetPermission instanceof DataSet_Source){
            $arrayAssoc = $dataSetPermission->getRows();
            if(isset($arrayAssoc['isValid'])){
                return $arrayAssoc['isValid'];
            }
        }
        
        $response->addDataSet(new DataSet("result", $msg));
    }
    
    public function validProductionProduct(DTO\Request\Row $request, DTO\Response $response) {
    	$parameters = $request->getParameters();
    	$row = $parameters['rowDadosRecipe'];

    	$sql = "SELECT A.TIPO, A.CDFILIAL, NVL(F.NMFILIAL,'') AS NMFILIAL, 
    				   NVL(A.CDSERVFILI,'') AS CDSERVFILI, NVL(S.NMSERVICO,'') AS NMSERVICO, 
    				   A.CDPRATO, P.NMPRATO, A.CDPRODPRODUCAO 
    			 FROM (SELECT 0 AS ID, 'P' AS TIPO, '' AS CDFILIAL, '' AS CDSERVFILI, 
    						  CDPRATO, CDFILAUXPRAT, CDPRODPRODUCAO 
    				     FROM PRATPADR 
    				    WHERE CDPRODPRODUCAO = :CDPRODPRODUCAO 
    				      AND ((:RECEITA <> 'P') OR ((:RECEITA = 'P') 
    				      AND (CDPRATO||CDFILAUXPRAT <> :CDPRATO||:CDFILAUXPRAT))) 
    				    UNION ALL 
    				   SELECT 1 AS ID, 'F' AS TIPO, CDFILIAL, '' AS CDSERVFILI, 
    				          CDPRATO, CDFILAUXPRAT, CDPRODPRODUCAO 
    				     FROM PRATFILI 
    				    WHERE CDPRODPRODUCAO = :CDPRODPRODUCAO 
    				      AND ((:RECEITA <> 'F') OR ((:RECEITA = 'F') 
    				      AND (CDPRATO||CDFILAUXPRAT <> :CDPRATO||:CDFILAUXPRAT))) 
    				    UNION ALL 
    				   SELECT 2 AS ID, 'S' AS TIPO, CDFILIAL, CDSERVFILI, 
    						  CDPRATO, CDFILAUXPRAT, CDPRODPRODUCAO 
    					 FROM PRATSERV 
    					WHERE CDPRODPRODUCAO = :CDPRODPRODUCAO 
    					  AND ((:RECEITA <> 'S') OR ((:RECEITA = 'S') 
    					  AND (CDPRATO||CDFILAUXPRAT <> :CDPRATO||:CDFILAUXPRAT)))
    				  ) A 
    			LEFT JOIN FILIAL F ON A.CDFILIAL = F.CDFILIAL 
    			LEFT JOIN SERVICO S ON S.CDFILIAL = A.CDFILIAL AND S.CDSERVFILI = A.CDSERVFILI 
    			LEFT JOIN PRATO P ON P.CDPRATO = A.CDPRATO AND P.CDFILAUXPRAT = A.CDFILAUXPRAT 
    		   ORDER BY A.ID, A.CDFILIAL, A.CDSERVFILI";
	
		$params = array(
			"CDPRATO"        => $row['CDPRATO'],
			"CDPRODPRODUCAO" => $row['CDPRODUTO'],
			"RECEITA"        => $row["RECEITA"],
			"CDFILAUXPRAT"   => $row["CDFILAUXPRAT"]
		);
		
		$stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    	$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
    }
    
    public function returnFatConversao(DTO\Request\Row $request, DTO\Response $response) {
    	$parameters = $request->getParameters();
    	$produtosRec = $parameters['prodReceita'];
    	$sgunidadeCompar = $parameters['sgUnidade'];
    	$qtAtiva = $parameters['qtAtiva'];
    	$nrorg = $this->utilities->getNrOrg();
    	$qtd = 0;
    	
    	foreach ($produtosRec as $produto) {
            $valorQtd = $produto[$qtAtiva];
            $sgUnidProd = $produto['SGUNIDADE'] ? $produto['SGUNIDADE'] : $produto['SGUNIDADEPROD'];
            if ($sgUnidProd != $sgunidadeCompar) {
                $fatorConv = $this->retornaFatorConversaoPrato($produto['CDPRODUTO'], $sgUnidProd, $sgunidadeCompar);
                if(!empty($fatorConv)) {
                    if ($fatorConv[0]['VRFATOCONVES'] === 0) {
                        $qtd = $qtd + $valorQtd;
                    } else {
                        $invertValor = ($fatorConv[0]['ORDEM'] === 3);
        
                        if ($fatorConv[0]['IDTIPOPE'] === '*') {
                            if ($invertValor === true) {
                                $qtd = $qtd + ($valorQtd / $fatorConv[0]['VRFATOCONVES']);
                            } else {
                                $qtd = $qtd + ($valorQtd * $fatorConv[0]['VRFATOCONVES']);
                            }
                        } else {
                            if ($invertValor === true) {
                                $qtd = $qtd + ($valorQtd * $fatorConv[0]['VRFATOCONVES']);
                            } else {
                                $qtd = $qtd + ($valorQtd / $fatorConv[0]['VRFATOCONVES']);
                            }
                        }
                    }
                }
            } else {
                $qtd = $qtd + $valorQtd;
            }
        }
        $response->addDataset(new DataSet('QTPESSOA', array($qtd)));
    }
    
    public function retornaFatorConversaoPrato($produto, $sgunidade, $sgunidadeCompar){
         $sql = "SELECT TO_NUMBER(C.VRFATOCONVES) AS VRFATOCONVES,
                        RPAD('/', 1, ' ')       AS IDTIPOPE,
                        0 AS ORDEM
                   FROM CONVPRODUTO C
                   LEFT JOIN PRODUTO P ON P.CDPRODUTO = C.CDPRODUTO
                  WHERE (C.CDPRODUTO = :P_CDPRODUTO)
                    AND (P.SGUNIDADE = :P_SGUNIDADEORI)
                    AND (C.SGUNIDADE = :P_SGUNIDADEDEST)

                  UNION ALL

                 SELECT TO_NUMBER(C.VRFATOCONVES) AS VRFATOCONVES,
                        RPAD('*', 1, ' ')       AS IDTIPOPE,
                        1 AS ORDEM
                   FROM CONVPRODUTO C
                   LEFT JOIN PRODUTO P ON P.CDPRODUTO = C.CDPRODUTO
                  WHERE (C.CDPRODUTO = :P_CDPRODUTO)
                    AND (P.SGUNIDADE = :P_SGUNIDADEDEST)
                    AND (C.SGUNIDADE = :P_SGUNIDADEORI)

                  UNION ALL

                 SELECT TO_NUMBER(VRFTCONVER)  AS VRFATOCONVES,
                        RPAD(IDTIPOPE, 1, ' ') AS IDTIPOPE,
                        2 AS ORDEM
                   FROM CONVUNID
                  WHERE (SGUNIDPARA = :P_SGUNIDADEDEST)
                    AND (SGUNIDDE = :P_SGUNIDADEORI)

                  UNION ALL

                 SELECT TO_NUMBER(VRFTCONVER)  AS VRFATOCONVES,
                        RPAD(IDTIPOPE, 1, ' ') AS IDTIPOPE,
                        3 AS ORDEM
                   FROM CONVUNID
                  WHERE (SGUNIDPARA = :P_SGUNIDADEORI )
                    AND (SGUNIDDE = :P_SGUNIDADEDEST )

                  ORDER BY ORDEM";

        $params = array(
            "P_CDPRODUTO" => $produto,
            "P_SGUNIDADEORI" => $sgunidade,
            "P_SGUNIDADEDEST" => $sgunidadeCompar
        );

        try {
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function saveRecipePlate(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $tipoReceita = $row['TIPORECEITA'];
        $dadosPratoDestino = '';

        switch ($tipoReceita) {
            case 'R':
                $dadosPratoDestino = 'pratregiao';
                if ($row['__is_new'] == true) $row['NRPRATREGIAO'] = $this->newCode->getCodeByTableName('PRATREGIAO', $this->environment->getNrOrg());
                $row['NRPRATREGIAO'] = intval($row['NRPRATREGIAO']);
                if (empty($row['QTPADRAOREGIAO']) == true && $row['QTPADRAOREGIAO'] !== 0) $row['QTPADRAOREGIAO'] = '';	
            break;
            case 'Q':
                $dadosPratoDestino = 'pratpadquali';
                if ($row['__is_new'] == true) $row['NRPRATPADQUALI'] = $this->newCode->getCodeByTableName('PRATPADQUALI', $this->environment->getNrOrg());
                $row['NRPRATPADQUALI'] = intval($row['NRPRATPADQUALI']);
                if (empty($row['QTPADRAOPADQ']) == true && $row['QTPADRAOPADQ'] !== 0) $row['QTPADRAOPADQ'] = '';	
            break;
            case 'P':
                $dadosPratoDestino = 'pratpadr';
                if (empty($row['QTPADRAOPRAT']) == true && $row['QTPADRAOPRAT'] !== 0) $row['QTPADRAOPRAT'] = '';	
            break;
            case 'S':
                $dadosPratoDestino = 'pratserv';
                if (empty($row['QTPADRAOSERV']) == true && $row['QTPADRAOSERV'] !== 0) $row['QTPADRAOSERV'] = '';	
            break;
            case 'L':
                $dadosPratoDestino = 'pratfili';
        		if (empty($row['QTPADRAOFILI']) == true && $row['QTPADRAOFILI'] !== 0) $row['QTPADRAOFILI'] = '';	
            break;
            case 'C':
                $dadosPratoDestino = 'prattpcard';
                if (empty($row['QTPADRAOPRATTP']) == true && $row['QTPADRAOPRATTP'] !== 0) $row['QTPADRAOPRATTP'] = '';	
            break;
        }
        
        $rows = $this->crudService->getRowsToSave($request, $dadosPratoDestino);
        $this->crudService->save($dadosPratoDestino, $rows);
    }
    
    public function returnPratpadr(DTO\Request\Row $request, DTO\Response $response) {
    	$parameters = $request->getParameters();
    	$plate = $parameters['plate'];
    	$filAux = $parameters['filAux'];
    	$dataSourceName = 'pratpadr';
    	
		$conditions = array(
			array(
				'columnName' => 'CDPRATO',
				'operator' => '=',
				'value' => $plate
			),
			array(
				'columnName' => 'CDFILAUXPRAT',
				'operator' => '=',
				'value' => $filAux
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        $response->addDataSet(new DataSet("columnsToReturn", $result));
    }
    
    public function returnPratfili(DTO\Request\Row $request, DTO\Response $response) {
    	$parameters = $request->getParameters();
    	$plate = $parameters['plate'];
    	$filAux = $parameters['filAux'];
    	$unity = $parameters['cdfilial'];
    	$dataSourceName = 'pratfili';
    	
		$conditions = array(
			array(
				'columnName' => 'CDPRATO',
				'operator' => '=',
				'value' => $plate
			),
			array(
				'columnName' => 'CDFILAUXPRAT',
				'operator' => '=',
				'value' => $filAux
			),
			array(
				'columnName' => 'CDFILIAL',
				'operator' => '=',
				'value' => $unity
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        $response->addDataSet(new DataSet("columnsToReturn", $result));
    }
    
    public function returnPratserv(DTO\Request\Row $request, DTO\Response $response) {
    	$parameters = $request->getParameters();
    	$plate = $parameters['plate'];
    	$filAux = $parameters['filAux'];
    	$unity = $parameters['cdfilial'];
    	$service = $parameters['cdserv'];
    	$dataSourceName = 'pratserv';
    	
		$conditions = array(
			array(
				'columnName' => 'CDPRATO',
				'operator' => '=',
				'value' => $plate
			),
			array(
				'columnName' => 'CDFILAUXPRAT',
				'operator' => '=',
				'value' => $filAux
			),
			array(
				'columnName' => 'CDFILIAL',
				'operator' => '=',
				'value' => $unity
			),
			array(
				'columnName' => 'CDSERVFILI',
				'operator' => '=',
				'value' => $service
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        $response->addDataSet(new DataSet("columnsToReturn", $result));
    }
    
    public function returnPrattpcard(DTO\Request\Row $request, DTO\Response $response) {
    	$parameters = $request->getParameters();
    	$plate = $parameters['plate'];
    	$filAux = $parameters['filAux'];
    	$tipocard = $parameters['tipocard'];
    	$central = $parameters['central'];
    	$dataSourceName = 'prattpcard';
    	
		$conditions = array(
			array(
				'columnName' => 'CDPRATO',
				'operator' => '=',
				'value' => $plate
			),
			array(
				'columnName' => 'CDFILAUXPRAT',
				'operator' => '=',
				'value' => $filAux
			),
			array(
				'columnName' => 'CDTIPOCARD',
				'operator' => '=',
				'value' => $tipocard
			),
			array(
				'columnName' => 'CDFILCENTPME',
				'operator' => '=',
				'value' => $central
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        $response->addDataSet(new DataSet("columnsToReturn", $result));
    }
    
    public function returnPratRegiao(DTO\Request\Row $request, DTO\Response $response) {
    	$parameters = $request->getParameters();
    	$plate = $parameters['plate'];
    	$filAux = $parameters['filAux'];
    	$padrQual = $parameters['padrQual'];
    	$regiao = $parameters['regiao'];
    	$dataSourceName = 'pratregiao';
    	
        $nrprat = $this->newCode->getCodeByTableName('PRATREGIAO', $this->environment->getNrOrg());

		$conditions = array(
			array(
				'columnName' => 'CDPRATO',
				'operator' => '=',
				'value' => $plate
			),
			array(
				'columnName' => 'CDFILAUXPRAT',
				'operator' => '=',
				'value' => $filAux
			),
			array(
				'columnName' => 'NRPADRAOQUALIDADE',
				'operator' => '=',
				'value' => $padrQual
			),
			array(
				'columnName' => 'CDREGIAO',
				'operator' => '=',
				'value' => $regiao
			),
			array(
				'columnName' => 'NRPRATREGIAO',
				'operator' => '=',
				'value' => $nrprat
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        $response->addDataSet(new DataSet("columnsToReturn", $result));
    }
    
    public function returnPratogramag(DTO\Request\Row $request, DTO\Response $response) {
    	$parameters = $request->getParameters();
    	$plate = $parameters['plate'];
    	$filAux = $parameters['filAux'];
    	$unity = $parameters['cdfilial'];
    	$service = $parameters['cdserv'];
    	$dataSourceName = 'pratogramag';
    	
        $conditions = array(
			array(
				'columnName' => 'CDPRATO',
				'operator' => '=',
				'value' => $plate
			),
			array(
				'columnName' => 'CDFILAUXPRAT',
				'operator' => '=',
				'value' => $filAux
			),
			array(
				'columnName' => 'CDFILIAL',
				'operator' => '=',
				'value' => $unity
			),
			array(
				'columnName' => 'CDSERVFILI',
				'operator' => '=',
				'value' => $service
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        $response->addDataSet(new DataSet("columnsToReturn", $result));
    }
    
    public function isValidAccess(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();
		$cdGrupOper = null;
		$perfAcess = null;
		$permission = true;
		
		$perfAcess = $this->getPerfilAcesso($row['AUTENTICACAO']['ALIAS_NAME']);
		
		if ($perfAcess !== NULL) { //se encontrar perfil de acesso
			$result = $this->validPerfilAcesso($perfAcess, $row['CDOPERADOR']); // valida PERFACESSPARC
			if (empty($result)) $permission = false;
		} else {
			$cdGrupOper = $this->getGrupOperParamGeral($row['AUTENTICACAO']['COLUMN_NAME']);
			if ($cdGrupOper) {
				$result = $this->validGrupOper($cdGrupOper, $row['CDOPERADOR']);
				if (empty($result) == true) $permission = false;
			}
		}

		if(!$cdGrupOper && !$perfAcess) $permission = true;
		
		$response->addDataSet(new DataSet ('permission', array('isValid' => $permission)));
	}
	
	public function getGrupOperAutgrupop($idautgrupop){
        $dataSourceName = 'autgrupop';
		$conditions = array(
			array(
				'columnName' => 'IDAUTGRUPOP',
				'operator' => 'LIKE',
				'value' => $idautgrupop
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		
		if(!empty($result)){
		    return $result[0]['CDGRUPOPER'];
		} else{
		    return null;
		}
    }
	
	public function getGrupOperParamGeral($columnName){
        $dataSourceName = 'paramgeral';
		$conditions = array();
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		
		if(!empty($result)){
		    return $result[0][$columnName];
		} else{
		    return null;
		}
    }
	
	public function authOperator(DTO\Request $request, DTO\Response $response) {
		try {
			$row = $request->getRow();
			$rowAuth = array(
	            "ALIAS_NAME"  => "ALTRECEPADRAO",
	            "COLUMN_NAME" => "CDGROPERREPA",
	            "CDOPERADOR"  => $this->environment->getCdOperador()
	        );
			$row['AUTENTICACAO'] = $rowAuth;
			$operator = $this->findCdOperator($row['EMAIL']);

			if(!empty($row['EMAIL'])){
				$senha = $row['SENHA'];
				$this->validatePassword($operator, $row['SENHA']);
			}

			$row['CDOPERADOR'] = $operator;
			$newRequest = new DTO\Request\Row($row, null, null, null);
			$this->isValidAccess($newRequest, $response);
		    
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function validatePassword($operator, $password) {
		$dataSourceName = 'operadores';
		$conditions = array(
			array(
				'columnName' => 'CDOPERADOR',
				'operator' => '=',
				"value" => $operator
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		if(md5($password) != $dataSet[0]['CDSENHAOPERWEB']){
			throw new \Exception("INVALID_PASSWORD", 1);
		}
	}
	
	public function findCdOperator($email) {
		$dataSourceName = 'operadores';
		$conditions = array(
			array(
				'columnName' => 'DSEMAILOPER',
				'operator' => 'LIKE',
				'value' => $email
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		if(empty($dataSet)){
			throw new \Exception("operadorNaoCadastrado", 1);
		} else {
			return $dataSet[0]['CDOPERADOR'];
		}
	}
	
	public function validationDependingGroup($request, $response){
		$row = $request->getRow();
		if (count($row) == 0) $row = $request->getParameters()['rowForm'];
		$cdGrupOper = null;
		$perfAcess = null;
		$permission = true;
		$perfAcess = $this->getPerfilAcesso($row['ALIAS_NAME']);
		
		if($perfAcess == NULL) { //não tem perfil e ai pesquisa parametrização por grupo
			if($row['COLUMN_NAME']) $cdGrupOper = $this->getGrupOperParamGeral($row['COLUMN_NAME']);
			else $cdGrupOper = $this->getGrupOperAutgrupop($row['ALIAS_NAME']);
		}
		
		if(!$cdGrupOper && !$perfAcess) {
			$str = 'NOGROUP';
			$response->addDataSet(new DataSet ('permission', array('result' => $str)));
		} else {
			if($perfAcess) $result = $this->validPerfilAcesso($perfAcess, $this->environment->getCdOperador());
			else $result = $this->validGrupOper($cdGrupOper, $this->environment->getCdOperador());
			
			$str = !empty($result) ? "VALID" : 'NOPERMISSION';
			$response->addDataSet(new DataSet ('permission', array('result' => $str)));
		}
	}

	public function getPerfilAcesso($idautperfacesso){
        $dataSourceName = 'autperfilacesso';
		$conditions = array(
			array(
				'columnName' => 'IDAUTPERFACESSO',
				'operator' => 'LIKE',
				'value' => $idautperfacesso
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();

		if(!empty($result)) return $result[0]['NRPERFILACESSO'];
		else return null;
    }

	public function validGrupOper($cdGrupOper, $cdOperador){
        $dataSourceName = 'opergrupop';
		$conditions = array(
			array(
				'columnName' => 'CDGRUPOPER',
				'operator' => '=',
				'value' => $cdGrupOper
			),
			array(
				'columnName' => 'CDOPERADOR',
				'operator' => '=',
				'value' => $cdOperador
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		return $result;
    }

	public function validPerfilAcesso($nrperfilacesso, $cdOperador){
        $dataSourceName = 'perfAcessParcOperador';
		$conditions = array(
			array(
				'columnName' => 'NRPERFILACESSO',
				'operator' => '=',
				'value' => $nrperfilacesso
			),
			array(
				'columnName' => 'CDOPERADOR',
				'operator' => '=',
				'value' => $cdOperador
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		return $result;
    }
    
    public function getResponseLastDataSet(DTO\Response $response) {
        return $response->getDataSets()[count($response->getDataSets()) - 1];
    }
    
    public function insertPratoGramag($params) {
        $insert = "INSERT INTO PRATOGRAMAG (CDFILAUXPRAT, CDPRATO, CDFILIAL, CDSERVFILI, IDORIGEMRECE, CDFILIALORI, 
                      CDSERVFILIORI, VRPERCENTUAL, NRORG)
                      VALUES (:CDFILAUXPRAT, :CDPRATO, :CDFILIAL, :CDSERVFILI, :IDORIGEMRECE,  :CDFILIALORI, 
                      :CDSERVFILIORI, :VRPERCENTUAL, :NRORG)";
        $params['NRORG'] = $this->utilities->getNrOrg();
        $this->connection->prepare($insert)->execute($params);
    }
    
    public function getImageBlob(DTO\Request\Row $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        
        try {
    		$sql = "SELECT BASE64ENCODE(FGFOTOPRATO) AS FGFOTOPRATO FROM FOTOPRATOBLOB FTB
    				WHERE FTB.CDPRATO = :CDPRATO
                    AND FTB.CDFILAUXPRAT = :CDFILAUXPRAT
                    AND FTB.IDRECEITA = :IDRECEITA
                    AND ((FTB.CDFILIAL IS NULL) OR (FTB.CDFILIAL = :CDFILIAL))
                    AND ((FTB.CDSERVFILI IS NULL) OR (FTB.CDSERVFILI = :CDSERVFILI))
                    AND NRFOTO = '01'
                    ";
    				// AND FGFOTOPRATO IS NOT NULL";

        	$query = $this->entityManager->getConnection()->prepare($sql);
	        $sqlParams = array(
	        	"CDPRATO" => !empty($params['CDPRATO']) ? $params['CDPRATO'] : null,
	        	"CDFILAUXPRAT" => !empty($params['CDFILAUXPRAT']) ? $params['CDFILAUXPRAT'] : null,
	        	"IDRECEITA" => !empty($params['IDRECEITA']) ? $params['IDRECEITA'] : null,
	        	"CDFILIAL" => !empty($params['CDFILIAL']) ? $params['CDFILIAL'] : null,
	        	"CDSERVFILI" => !empty($params['CDSERVFILI']) ? $params['CDSERVFILI'] : null,
	        );
	        $query->execute($sqlParams);
	        $image = $query->fetchAll();
	        if (count($image) > 0) {
	            $code_base64 = $image[0]["FGFOTOPRATO"];
	            $code_base64 = str_replace('data:image/jpeg;base64,', '', $code_base64);
	            $code_binary = base64_decode($code_base64);
	            $image = imagecreatefromstring($code_binary);
	            header('Content-Type: image/jpeg');
	            header('Cache-Control: public, max-age=31536000');
	            header('Access-Control-Allow-Origin: *');
	            imagejpeg($image);
	            imagedestroy($image);
	            die;
	        } else {
	            http_response_code(404);
	        }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getImage(DTO\Request\Row $request, DTO\Response $response)
    {
    	try {
    		$row = $request->getRow();
    		
    		$sql = "SELECT FTB.NRFOTOPRATOBLOB,FTB.CDPRATO,FTB.CDFILAUXPRAT,FTB.IDRECEITA, :FILESERVER || FTB.FGFOTOPRATOURL AS FGFOTOMOBILE, 
    					FGFOTOPRATOURL, CASE WHEN FTB.FGFOTOPRATO IS NULL THEN 'N' ELSE 'S' END AS HASFGFOTOPRATO
    				FROM FOTOPRATOBLOB FTB
	                WHERE FTB.CDPRATO = :CDPRATO
	                    AND FTB.CDFILAUXPRAT = :CDFILAUXPRAT
	                    AND FTB.IDRECEITA = :IDRECEITA
	                    AND ((FTB.CDFILIAL IS NULL) OR (FTB.CDFILIAL = :CDFILIAL))
	                    AND ((FTB.CDSERVFILI IS NULL) OR (FTB.CDSERVFILI = :CDSERVFILI))
	                    AND ((FTB.CDFILCENTPME IS NULL) OR (FTB.CDFILCENTPME = :CDFILCENTPME))
	                    AND ((FTB.CDTIPOCARD IS NULL) OR (FTB.CDTIPOCARD = :CDTIPOCARD))
	                    AND NRFOTO = '01'";
	        if (property_exists($row, 'NRFOTOPRATOBLOB') && !empty($row['NRFOTOPRATOBLOB'])) $sql .= " AND NRFOTOPRATOBLOB = :NRFOTOPRATOBLOB";
	        $sql .= " ORDER BY FGFOTOPRATOURL";
	        
			$params = array(
			    "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
			    "CDPRATO" => $row['CDPRATO'],
			    "IDRECEITA" => $row['IDRECEITA'],
			    "FILESERVER" => $this->imgPathURL,
			    "CDFILIAL" => $row['CDFILIAL'],
			    "CDSERVFILI" => $row['CDSERVFILI'],
			    "CDFILCENTPME" => $row['CDFILCENTPME'],
			    "CDTIPOCARD" => $row['CDTIPOCARD'],
			);
			
	        $result = $this->entityManager->getConnection()->fetchAssoc($sql, $params);
	    	if (!empty($result)) $response->addDataSet(new DataSet ('image', $result));
	    	else $response->addDataSet(new DataSet ('image', []));
    	} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
    }
    
    public function deleteImage(DTO\Request $request, DTO\Response $response) {
		$this->connection->beginTransaction();
		try {
			$row = $request->getRow();
			
			$this->getImage($request, $response);
			$image = $this->getResponseLastDataSet($response)->getRows();
			
			if (!empty($image["FGFOTOPRATOURL"])) $this->fileUploader->deleteFile($image['FGFOTOPRATOURL']);
		
			if (!empty($image['FGFOTOPRATOURL']) && $image['HASFGFOTOPRATO'] === "S") {
				$sql = "UPDATE FOTOPRATOBLOB SET FGFOTOPRATOURL = :FGFOTOPRATOURL, DTULTATU = :DTULTATU
							WHERE CDPRATO = :CDPRATO
		                                AND CDFILAUXPRAT = :CDFILAUXPRAT
		                                AND IDRECEITA = :IDRECEITA
		                                AND ((CDFILIAL IS NULL) OR (CDFILIAL = :CDFILIAL))
		                                AND ((CDSERVFILI IS NULL) OR (CDSERVFILI = :CDSERVFILI))
		                                AND ((CDFILCENTPME IS NULL) OR (CDFILCENTPME = :CDFILCENTPME))
		                                AND ((CDTIPOCARD IS NULL) OR (CDTIPOCARD = :CDTIPOCARD))
		                                AND NRFOTOPRATOBLOB = :NRFOTOPRATOBLOB";
				$params = array(
				    "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
				    "CDPRATO" => $row['CDPRATO'],
				    "IDRECEITA" => $row['IDRECEITA'],
				    "FGFOTOPRATOURL" => null,
				    "CDFILIAL" => $row['CDFILIAL'],
				    "CDSERVFILI" => $row['CDSERVFILI'],
				    "CDFILCENTPME" => $row['CDFILCENTPME'],
				    "CDTIPOCARD" => $row['CDTIPOCARD'],
				    "DTULTATU"    => date('d/m/Y'),
				    "NRFOTOPRATOBLOB" => $row['NRFOTOPRATOBLOB'],
				);
				
				$query = $this->entityManager->getConnection()->prepare($sql);
		        $query->execute($params);
			} else {
				// $sql = "UPDATE FOTOPRATOBLOB SET FGFOTOPRATOURL = :FGFOTOPRATOURL, DTULTATU = :DTULTATU
				$sql = "DELETE FROM FOTOPRATOBLOB
						WHERE CDPRATO = :CDPRATO
	                                AND CDFILAUXPRAT = :CDFILAUXPRAT
	                                AND IDRECEITA = :IDRECEITA
	                                AND ((CDFILIAL IS NULL) OR (CDFILIAL = :CDFILIAL))
	                                AND ((CDSERVFILI IS NULL) OR (CDSERVFILI = :CDSERVFILI))
	                                AND ((CDFILCENTPME IS NULL) OR (CDFILCENTPME = :CDFILCENTPME))
	                                AND ((CDTIPOCARD IS NULL) OR (CDTIPOCARD = :CDTIPOCARD))
	                                AND NRFOTOPRATOBLOB = :NRFOTOPRATOBLOB";
				$params = array(
				    "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
				    "CDPRATO" => $row['CDPRATO'],
				    "IDRECEITA" => $row['IDRECEITA'],
				    // "FGFOTOPRATOURL" => null,
				    "CDFILIAL" => $row['CDFILIAL'],
				    "CDSERVFILI" => $row['CDSERVFILI'],
				    "CDFILCENTPME" => $row['CDFILCENTPME'],
				    "CDTIPOCARD" => $row['CDTIPOCARD'],
				    // "DTULTATU"    => date('d/m/Y')
				    "NRFOTOPRATOBLOB" => $row['NRFOTOPRATOBLOB'],
				);
				
				$query = $this->entityManager->getConnection()->prepare($sql);
		        $query->execute($params);
		        
		        if ($row["IDRECEITA"] !== 'T') {
			        if ($row["IDRECEITA"] === 'P') {
			        	$deleteFoto = "DELETE FROM FOTOPRAPAD
							WHERE CDPRATO = :CDPRATO
		                                AND CDFILAUXPRAT = :CDFILAUXPRAT
		                                AND NRFOTOPRAPAD = '01'";
						$paramsDel = array(
						    "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
						    "CDPRATO" => $row['CDPRATO'],
						);
			        } else if ($row["IDRECEITA"] === 'F') {
			        	
			        	$deleteFoto = "DELETE FROM FOTOPRAFIL
							WHERE CDPRATO = :CDPRATO
		                                AND CDFILAUXPRAT = :CDFILAUXPRAT
		                                AND CDFILIAL = :CDFILIAL
		                                AND NRFOTOPRAFIL = '01'";
						$paramsDel = array(
						    "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
						    "CDPRATO" => $row['CDPRATO'],
						    "CDFILIAL" => $row['CDFILIAL'],
						);
			        } else if ($row["IDRECEITA"] === 'S') {
			        	$deleteFoto = "DELETE FROM FOTOPRASER 
							WHERE CDPRATO = :CDPRATO
		                                AND CDFILAUXPRAT = :CDFILAUXPRAT
		                                AND CDFILIAL = :CDFILIAL
		                                AND CDSERVFILI = :CDSERVFILI
		                                AND NRFOTOPRASER = '01'";
						$paramsDel = array(
						    "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
						    "CDPRATO" => $row['CDPRATO'],
						    "CDFILIAL" => $row['CDFILIAL'],
						    "CDSERVFILI" => $row['CDSERVFILI'],
						);
			        }
			        
			        $query = $this->entityManager->getConnection()->prepare($deleteFoto);
			        $query->execute($paramsDel);
		        }
			}
			
	    	$this->connection->commit();
	        $response->addDataSet(new DataSet ('deleteImage', array()));
		} catch (\Exception $e) {
			$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function saveImage(DTO\Request $request, DTO\Response $response){
        try{
        	
            $this->connection->beginTransaction();
            $row = $request->getRow();
          
            $cdfilial = isset($row['CDFILIAL']) ? $row['CDFILIAL'] : null;
            $cdservfili = isset($row['CDSERVFILI']) ? $row['CDSERVFILI'] : null;
            $cdfilcentpme = isset($row['CDFILCENTPME']) ? $row['CDFILCENTPME'] : null;
            $cdtipocard = isset($row['CDTIPOCARD']) ? $row['CDTIPOCARD'] : null;
            
            if (isset($row["IMAGE_UPLOAD"]) && isset($row["IMAGE_UPLOAD"]["0"]) 
    		&& isset($row["IMAGE_UPLOAD"]["0"]["path"]) && $row["IMAGE_UPLOAD"]["0"]["path"] !== null) {
    			$row["FGFOTOPRAPAD"] = $row["IMAGE_UPLOAD"]["0"]["path"];
    		}
 			
    		$nrfotopratoblob =  $this->newCode->getCodeByTableName('FOTOPRATOBLOB', $this->environment->getNrOrg());
    		
    		$verificaRegistoFoto = $this->verifyImage($row,$cdfilial,$cdservfili, $cdfilcentpme, $cdtipocard);
			
            if (empty($verificaRegistoFoto)) {
                
                $insert = "INSERT INTO FOTOPRATOBLOB (CDFILAUXPRAT, CDPRATO,  IDRECEITA, FGFOTOPRATOURL, NRFOTOPRATOBLOB, NRFOTO, CDFILIAL, CDSERVFILI, DTINCLUSAO, DTULTATU, CDOPERINCLUSAO, CDOPERULTATU, CDFILCENTPME, CDTIPOCARD)
                            VALUES (:CDFILAUXPRAT, :CDPRATO,  :IDRECEITA, :FGFOTOPRATOURL, :NRFOTOPRATOBLOB, :NRFOTO, :CDFILIAL, :CDSERVFILI, :DTINCLUSAO, :DTULTATU, :CDOPERADOR, :CDOPERADOR, :CDFILCENTPME, :CDTIPOCARD)";

                $params = array(
                    "CDFILAUXPRAT"     => $row['CDFILAUXPRAT'],
                    "CDPRATO"          => $row['CDPRATO'],
                    "IDRECEITA"        => $row['IDRECEITA'],
                    "FGFOTOPRATOURL"   => $row['FGFOTOPRAPAD'],
                    "NRFOTOPRATOBLOB"  => $nrfotopratoblob,
                    "NRFOTO"           => '01', // No Delphi é possivél salvar mais de uma foto, no mobile ficou fixo por salvar apenas uma foto por prato.
                    "CDFILIAL"         => $cdfilial,
                    "CDSERVFILI"       => $cdservfili,
                    "DTINCLUSAO"       => date('d/m/Y'),
                    "DTULTATU"         => date('d/m/Y'),
                    "CDOPERADOR"	   => $this->environment->getCdOperador(),
                    "CDFILCENTPME"     => $cdfilcentpme,
			    	"CDTIPOCARD"       => $cdtipocard,
                );

                $stmt = $this->connection->prepare($insert);
                $stmt->execute($params);
                
            } else if (!empty($verificaRegistoFoto)) {
                
                $update = "UPDATE FOTOPRATOBLOB SET FGFOTOPRATOURL = :FGFOTOPRATOURL, DTULTATU = :DTULTATU, CDOPERULTATU = :CDOPERULTATU
                            WHERE CDPRATO = :CDPRATO
                                AND CDFILAUXPRAT = :CDFILAUXPRAT
                                AND IDRECEITA = :IDRECEITA
                                AND ((CDFILIAL IS NULL) OR (CDFILIAL = :CDFILIAL))
                                AND ((CDSERVFILI IS NULL) OR (CDSERVFILI = :CDSERVFILI))
                                AND ((CDFILCENTPME IS NULL) OR (CDFILCENTPME = :CDFILCENTPME))
                                AND ((CDTIPOCARD IS NULL) OR (CDTIPOCARD = :CDTIPOCARD))";

        
                $params = array(
                    "FGFOTOPRATOURL"   => $row['FGFOTOPRAPAD'],
                    "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
        		    "CDPRATO" => $row['CDPRATO'],
        		    "IDRECEITA" => $row['IDRECEITA'],
        		    "CDFILIAL" => $cdfilial,
        		    "CDSERVFILI" => $cdservfili,
        		    "DTULTATU"    => date('d/m/Y'),
                   "CDOPERULTATU" => $this->environment->getCdOperador(),
                   "CDFILCENTPME" => $cdfilcentpme,
                   "CDTIPOCARD"   => $cdtipocard,
                );
                
                $query = $this->connection->prepare($update);
                $query->execute($params);
                
            }
            
            $this->connection->commit();
        }catch(\Exception $e){
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function verifyImage($row,$cdfilial,$cdservfili, $cdfilcentpme, $cdtipocard){
        
        $sql = "SELECT MAX (NRFOTO) AS FOTO, FTB.CDFILIAL, FTB.CDSERVFILI, FTB.CDPRATO, FTB.IDRECEITA FROM FOTOPRATOBLOB FTB
                
                WHERE FTB.CDPRATO = :CDPRATO
                    AND FTB.CDFILAUXPRAT = :CDFILAUXPRAT
                    AND FTB.IDRECEITA = :IDRECEITA
                    AND ((FTB.CDFILIAL IS NULL) OR (FTB.CDFILIAL = :CDFILIAL))
                    AND ((FTB.CDSERVFILI IS NULL) OR (FTB.CDSERVFILI = :CDSERVFILI))
                    AND ((FTB.CDFILCENTPME IS NULL) OR (FTB.CDFILCENTPME = :CDFILCENTPME))
                    AND ((FTB.CDTIPOCARD IS NULL) OR (FTB.CDTIPOCARD = :CDTIPOCARD))
                GROUP BY FTB.CDFILIAL, FTB.CDSERVFILI, FTB.CDPRATO,FTB.IDRECEITA ";
		$params = array(
		    "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
		    "CDPRATO" => $row['CDPRATO'],
		    "IDRECEITA" => $row['IDRECEITA'],
		    "CDFILIAL" => $cdfilial,
		    "CDSERVFILI" => $cdservfili,
		    "CDFILCENTPME" => $cdfilcentpme,
		    "CDTIPOCARD" => $cdtipocard,
		);

		$query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        
        return !empty($result) ? $result[0] : null ;
        
    }
}
