import {
  Loader, Http, IEventParam, Metadata, IDictionary, dayjs,
  I18n,
} from '@zeedhi/core';
import {
  Select, Form, Date as ZdDate, GridEditable, LoadingService, SelectMultiple, IComponentRender, DialogService,
  TextInput,
} from '@zeedhi/common';

export class Pme05900AssistenteIntegracaoCardapio {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public crudController = Loader.getInstance('CrudController');
  
  public editable = false;
  
  private filterGridDestino = {};
  
  // private lastDataGridDestino: IDictionary[] = [];

  // public validaCentralIntegracaoCard() {
  //   const row = {
  //     CDFILCENTPME: this.appController.getCdCentral(),
  //   };
  //   LoadingService.show(' ');
  //   Http.post('/validaCentralIntegracaoCard', { requestType: 'Row', row }).then((response) => {
  //     const result = response.data.dataset.result;
  //     if (result.length > 0) {
  //       Http.post('/dataProcessamentoCentral', { requestType: 'Row', row }).then((value) => {
  //         const dataProcessamentoCentral = value.data.dataset.result[0];
  //         if (dataProcessamentoCentral < this.utilities.dtProcessFormatada) {
  //           this.utilities.showAlert('alertError', 'Data de processamento:', 'red');
  //         }
  //       });
  //     } else this.utilities.showAlert('alertError', 'Central não informada.', 'red');
  //   }).finally(() => {
  //     LoadingService.hide();
  //   });
  // }
  
  public setCentral({ component }: IEventParam<Select>) {
    let campo: Select;
    if (component.name === 'CENTRAL_ORIGEM') {
      campo = Metadata.getInstance('CENTRAL_ORIGEM');
    } else {
      campo = Metadata.getInstance('CENTRAL_DESTINO');
    }
    
    campo.setValue(this.appController.getCdCentral());
  }
  
  public clearDependentField({ component }: IDictionary) {
    const grupTpCard: Select = Metadata.getInstance('CDGRUPTIPOCARD');
    const tpCard: SelectMultiple = Metadata.getInstance('TPCARD');
    const servico: SelectMultiple = Metadata.getInstance('CDSERVFILI');

    if (component.name === 'CENTRAL_ORIGEM') {
      grupTpCard.setValue(null);
      tpCard.setValue(null);
      servico.setValue(null);
    } else if (component.name === 'CDGRUPTIPOCARD') {
      tpCard.setValue(null);
      servico.setValue(null);
    } else if (component.name === 'TPCARD' || component.name === 'GRUPOSERVICO') {
      servico.setValue(null);
    } else if (component.name === 'DTDE') {
      const finalDate: ZdDate = Metadata.getInstance('DTATE');
      finalDate.value = null;
    }
    
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    grid.datasource.data = [];
    grid.datasource.filter = [];
  }
  
  public filtraGrupTipoCard({ component }: IEventParam<Select>) {
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const centralOrigem = formOrigem.value.CENTRAL_ORIGEM;
    const filter = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralOrigem,
      },
      {
        name: 'P_CDGRUPTIPOCARD',
        operator: '=',
        value: 'T',
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public filtraTpCard({ component }: IEventParam<SelectMultiple>) {
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const centralOrigem = formOrigem.value.CENTRAL_ORIGEM;
    const grupoTpCard = formOrigem.value.CDGRUPTIPOCARD;
    const filter = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralOrigem,
      },
      {
        name: 'P_CDAGRUPTPCARD',
        operator: '=',
        value: grupoTpCard === null ? 'T' : grupoTpCard,
      },
    ];
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public dataAtual({ component }: IEventParam<ZdDate>) {
    component.value = dayjs().format('DD/MM/YYYY');
  }
  
  public setFilterService({ component }: IEventParam<SelectMultiple>) {
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const centralOrigem = formOrigem.value.CENTRAL_ORIGEM;
    const grupoTpCard = formOrigem.value.CDGRUPTIPOCARD;
    const tipoCard = Metadata.getInstance<SelectMultiple>('TPCARD').value;
    const grupoServ = formOrigem.value.GRUPOSERVICO;
    
    const conditions = [
      {
        name: 'P_LISTATPCARD',
        operator: '=',
        value: tipoCard || null,
      },
      {
        name: 'P_TODOSTPCARD',
        operator: '=',
        value: tipoCard.length > 0 ? tipoCard.join('_') : 'T',
      },
      {
        name: 'P_CDGRUPTIPOCARD',
        operator: '=',
        value: grupoTpCard || null,
      },
      {
        name: 'P_TGRUPTIPOCARD',
        operator: '=',
        value: grupoTpCard !== null ? grupoTpCard : 'T',
      },
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralOrigem,
      },
      {
        name: 'P_CDGRUPSERV',
        operator: '=',
        value: grupoServ !== null ? grupoServ.CDGRUPSERV : null,
      },
      {
        name: 'P_TCDGRUPSERV',
        operator: '=',
        value: grupoServ !== null ? grupoServ.CDGRUPSERV : 'T',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public filtraGridDestino() {
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    
    const cdServFili = (formOrigem.value.CDSERVFILI.length > 0) ? formOrigem.value.CDSERVFILI.join('_') : 'T';
    
    let cdGrupTipoCard = formOrigem.value.CDGRUPTIPOCARD;
    if (cdGrupTipoCard === null || cdGrupTipoCard === undefined) cdGrupTipoCard = 'T';
    
    const cdTipoCard = (formOrigem.value.TPCARD.length > 0) ? formOrigem.value.TPCARD.join('_') : 'T';
    
    const conditions = [
      {
        name: 'P_DATAFIN',
        operator: '=',
        value: formOrigem.value.DTATE,
      },
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: formOrigem.value.CENTRAL_ORIGEM,
      },
      {
        name: 'P_CDSERVFILI',
        operator: '=',
        value: cdServFili,
      },
      {
        name: 'P_DATAINI',
        operator: '=',
        value: formOrigem.value.DTDE,
      },
      {
        name: 'P_CDAGRUPTPCARD',
        operator: '=',
        value: cdGrupTipoCard,
      },
      {
        name: 'P_CDTIPOCARD',
        operator: '=',
        value: cdTipoCard,
      },
      {
        name: 'P_CDFILCENTPME_DEST',
        operator: '=',
        value: formCentralDest.value.CENTRAL_DESTINO,
      },
    ];
    
    grid.datasource.filter = conditions;
    grid.datasource.get();
  }
  
  public askReloadButton() {
    DialogService.show({
      name: 'reload',
      type: 'warning',
      title: 'Atenção',
      text: 'Ao prosseguir, os dados inseridos serão perdidos. Deseja continuar?',
      buttons: [
        {
          name: 'sim',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              this.reloadGridDestino();
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'nao',
          component: 'ZdButton',
          label: 'Não',
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }
  
  public reloadGridDestino() {
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const dataDe = formOrigem.value.DTDE;
    const dataAte = formOrigem.value.DTATE;
    
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    // this.lastDataGridDestino = grid.datasource.data;
    grid.datasource.data = [];
    grid.datasource.filter = [];
    this.gridEditController.cancelEditing({ component: grid });
    
    if (formOrigem.validate() && formCentralDest.validate()) {
      if (this.utilities.dateIsValid(dataDe, dataAte)) {
        this.setGridDestino();
      } else this.utilities.showAlert('invalidDate', 'Data inicial maior que a data final', 'error');
    }
  }
  
  public validaData() {
    const dataDe: ZdDate = Metadata.getInstance('DTDE');
    const dataAte: ZdDate = Metadata.getInstance('DTATE');
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    if (dataAte.value !== null && !this.utilities.dateIsValid(dataDe.value, dataAte.value)) {
      this.utilities.showAlert('invalidDate', 'Data inicial maior que a data final', 'error');
      dataAte.value = null;
      grid.datasource.data = [];
    } else {
      this.reloadGridDestino();
    }
  }
  
  public cancelEditingGridDest({ component }: IDictionary) {
    this.editable = false;
    const gridDestino = this.utilities.returnGrid(component) as GridEditable;
    if (gridDestino.editing) {
      this.gridEditController.newRow = false;
      this.gridEditController.cancelEditedRows({ component: gridDestino });
      gridDestino.datasource.currentRow = {};
      gridDestino.datasource.get();
    }
  }
  
  public addRow({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const row = grid.datasource.currentRow;

    Http.post('/returnDadosPadraoServ', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset.result;
      if (result.length > 0) this.gridEditController.addRow({ component: grid });
      else this.gridEditController.addRowSetFocusSecondPosition({ component: grid });
    });
  }
  
  public filtraTpCardDest() {
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const centralDest = formCentralDest.value.CENTRAL_DESTINO;
    
    return [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralDest,
      },
      {
        name: 'P_CDAGRUPTPCARD',
        operator: '=',
        value: 'T',
      },
    ];
  }
  
  public filtraServicoDestino() {
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const centralDest = formCentralDest.value.CENTRAL_DESTINO;
    
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const datasource = gridDestino.datasource.currentRow;
    
    return [
      {
        name: 'P_LISTATPCARD',
        operator: '=',
        value: datasource.TPCARD_DESTINO,
      },
      {
        name: 'P_TODOSTPCARD',
        operator: '=',
        value: 'T',
      },
      {
        name: 'P_CDGRUPTIPOCARD',
        operator: '=',
        value: null,
      },
      {
        name: 'P_TGRUPTIPOCARD',
        operator: '=',
        value: 'T',
      },
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: centralDest,
      },
      {
        name: 'P_CDGRUPSERV',
        operator: '=',
        value: null,
      },
      {
        name: 'P_TCDGRUPSERV',
        operator: '=',
        value: 'T',
      },
    ];
  }

  public askFieldReplicationPer(fields: { [key: string]: any }) {
    DialogService.show({
      name: 'replicarValor',
      type: 'warning',
      title: 'Atenção',
      text: 'desejaRepetirLinhas',
      buttons: [
        {
          name: 'acrescentar',
          component: 'ZdButton',
          label: 'Acrescentar',
          events: {
            click: () => {
              this.replicateFieldValueMultiplePeriodo('A', fields);
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'sobrepor',
          component: 'ZdButton',
          label: 'Sobrepor',
          events: {
            click: () => {
              this.replicateFieldValueMultiplePeriodo('S', fields);
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }
  
  public replicateFieldValueMultiplePeriodo(option: string, fields: { [key: string]: any }) {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const dataProcessamento = dayjs(this.utilities.dtProcessFormatada, this.utilities.getFormatLocale());
  
    const periodoDeDestinoInicio = dayjs(fields.DTDE_DESTINO, this.utilities.getFormatLocale());
    const editedRows = gridDestino.getEditedRows(false, true);
    const editedRowsMap = new Map(editedRows.map((row) => [row.PK, row]));
  
    gridDestino.datasource.data.forEach((row) => {
      if (row.PK !== gridDestino.datasource.currentRow.PK) {
        const periodoDeOrigemInicio = dayjs(row.DTINICARDPME, this.utilities.getFormatLocale());
        const periodoDeOrigemFim = dayjs(row.DTFINCARDPME, this.utilities.getFormatLocale());
  
        // const diffInMs = periodoDeOrigemFim.getTime() - periodoDeOrigemInicio.getTime();
        // const diffInDays = Math.ceil(diffInMs / (1000 * 60 * 60 * 24));
        const diffInDays = periodoDeOrigemFim.diff(periodoDeOrigemInicio, 'day');
  
        // const periodoAteDestino = new Date(periodoDeDestinoInicio);
        // periodoAteDestino.setDate(periodoAteDestino.getDate() + diffInDays);
        const periodoAteDestino = periodoDeDestinoInicio.add(diffInDays, 'day');
  
        if (periodoAteDestino < dataProcessamento) {
          this.utilities.showAlert('alert', 'Data deve ser menor ou igual a data de processamento.', 'warning');
          gridDestino.updateCell(row.PK, 'DTATE_DESTINO', null);
          gridDestino.updateCell(row.PK, 'DTATE_DESTINO', '');
        } else {
          const formattedDestino = periodoAteDestino.format(this.utilities.getFormatLocale());
          const formattedInicio = periodoDeDestinoInicio.format(this.utilities.getFormatLocale());
  
          if (option === 'S') {
            gridDestino.updateCell(row.PK, 'DTATE_DESTINO', formattedDestino);
            gridDestino.updateCell(row.PK, 'DTDE_DESTINO', formattedInicio);
          } else if (option === 'A') {
            const editedRow = editedRowsMap.get(row.PK);
            const dtAteDestinoValue = editedRow ? editedRow.DTATE_DESTINO : row.DTATE_DESTINO;
  
            if (dtAteDestinoValue === null || dtAteDestinoValue === undefined || dtAteDestinoValue === '') {
              gridDestino.updateCell(row.PK, 'DTATE_DESTINO', formattedDestino);
              gridDestino.updateCell(row.PK, 'DTDE_DESTINO', formattedInicio);
            }
          }
        }
      }
    });
  }
  
  public validaDiaSemanaDestEOrigem({ component }: IEventParam<ZdDate>) {
    const diaSemanaDestino = dayjs(component.value, this.utilities.getFormatLocale()).format('ddd');
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const currentRow = gridDestino.datasource.currentRow;
    const diaSemanaOrigem = dayjs(currentRow.DTINICARDPME, this.utilities.getFormatLocale()).format('ddd');
    const editedRows = gridDestino.getEditedRows(false, true);
    
    editedRows.forEach((row: any) => {
      if (row.PK === currentRow.PK) { // Verifica se é a linha sendo editada
        this.buttons();
        const dtDeDestino = dayjs(row.DTDE_DESTINO, this.utilities.getFormatLocale());
        const dtAteDestino = dayjs(row.DTATE_DESTINO, this.utilities.getFormatLocale());
  
        // Verifica se a data inicial de destino está dentro do período de destino
        const dtPerIniDestino = row.DTPERINI_DESTINO
          ? dayjs(row.DTPERINI_DESTINO, this.utilities.getFormatLocale()) : null;
        
        if (dtPerIniDestino && dtDeDestino && dtAteDestino) {
          if (dtPerIniDestino >= dtDeDestino && dtPerIniDestino <= dtAteDestino) {
            if (diaSemanaDestino !== diaSemanaOrigem) {
              gridDestino.updateCell(currentRow.PK, 'DTPERINI_DESTINO', null);
              gridDestino.updateCell(currentRow.PK, 'DIASEMANA_DESTINO', null);
              this.utilities.showAlert(
                'alert',
                'Dia da semana que inicia o período de destino deve ser igual ao '
                + 'dia da semana que inicia o período de origem.',
                'warning',
              );
            } else {
              this.setDiaSemanaDest(gridDestino, diaSemanaDestino, component.value);
            }
          } else {
            gridDestino.updateCell(currentRow.PK, 'DTPERINI_DESTINO', undefined);
            gridDestino.updateCell(currentRow.PK, 'DIASEMANA_DESTINO', undefined);
            component.value = null;
            this.buttons();
            this.utilities.showAlert('alert', 'pme::dataIncialNaoCompreendePeriodo', 'warning');
          }
        } else {
          gridDestino.updateCell(currentRow.PK, 'DTPERINI_DESTINO', undefined);
          gridDestino.updateCell(currentRow.PK, 'DIASEMANA_DESTINO', undefined);
          component.value = null;
          this.buttons();
          this.utilities.showAlert('alert', 'pme::dataIncialNaoCompreendePeriodo', 'warning');
        }
      }
    });
  }
  
  public setDiaSemanaDest(grid: GridEditable, dia: string, dataInicial: string) {
    grid.updateCell(grid.datasource.currentRow.PK, 'DIASEMANA_DESTINO', dia !== '' ? dia : null);
    if (dia !== '' && grid.datasource.data.length > 1) {
      this.askFieldReplication({
        DIASEMANA_DESTINO: dia,
        DTPERINI_DESTINO: dataInicial,
      });
    }
  }

  public askFieldReplication(fields: { [key: string]: any }) {
    DialogService.show({
      name: 'replicarValor',
      type: 'warning',
      title: 'Atenção',
      text: 'desejaRepetirLinhas',
      buttons: [
        {
          name: 'acrescentar',
          component: 'ZdButton',
          label: 'Acrescentar',
          events: {
            click: () => {
              this.replicateFieldValueMultiple('A', fields);
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'sobrepor',
          component: 'ZdButton',
          label: 'Sobrepor',
          events: {
            click: () => {
              this.replicateFieldValueMultiple('S', fields);
              DialogService.hide();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public replicateFieldValueMultiple(option: string, fields: { [key: string]: any }) {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const editedRows = gridDestino.getEditedRows(false, true);
    
    if ('DSCARDPME_DESTINO' in fields) {
      const editedRowsMap = new Map(editedRows.map((row) => [row.PK, row]));
      gridDestino.datasource.data.forEach((row) => {
        const pk = row.PK;
        const editedRow = editedRowsMap.get(pk);
  
        if (option === 'S') {
          gridDestino.updateCell(pk, 'DSCARDPME_DESTINO', fields.DSCARDPME_DESTINO);
        } else if (option === 'A') {
          const descricao = editedRow ? editedRow.DSCARDPME_DESTINO : row.DSCARDPME_DESTINO;
          if (descricao === null || descricao === '') {
            gridDestino.updateCell(pk, 'DSCARDPME_DESTINO', fields.DSCARDPME_DESTINO);
          }
        }
      });
    } else {
      editedRows.forEach((row: any) => {
        const pk = row.PK;
        const dtDeDestino = dayjs(row.DTDE_DESTINO, this.utilities.getFormatLocale());
        const dtAteDestino = dayjs(row.DTATE_DESTINO, this.utilities.getFormatLocale());
        const diaSemanaOrigem = row.DIASEMANA;
  
        // Verifica se a data inicial de destino está dentro do período de destino
        const dtPerIniDestino = dayjs(fields.DTPERINI_DESTINO, this.utilities.getFormatLocale());
        if (dtPerIniDestino >= dtDeDestino && dtPerIniDestino <= dtAteDestino) {
          // Verifica se o dia da semana de destino é o mesmo que o da origem
          const diaSemanaDestino = fields.DIASEMANA_DESTINO;
          if (diaSemanaOrigem === diaSemanaDestino) {
            if (option === 'S') {
              gridDestino.updateCell(pk, 'DTPERINI_DESTINO', fields.DTPERINI_DESTINO);
              gridDestino.updateCell(pk, 'DIASEMANA_DESTINO', fields.DIASEMANA_DESTINO);
            } else if (option === 'A') {
              if (row.DTPERINI_DESTINO === null || row.DTPERINI_DESTINO === '') {
                gridDestino.updateCell(pk, 'DTPERINI_DESTINO', fields.DTPERINI_DESTINO);
                gridDestino.updateCell(pk, 'DIASEMANA_DESTINO', fields.DIASEMANA_DESTINO);
              }
            }
          }
        } else {
          gridDestino.updateCell(pk, 'DTPERINI_DESTINO', null);
          gridDestino.updateCell(pk, 'DTPERINI_DESTINO', '');
          gridDestino.updateCell(pk, 'DTPERINI_DESTINO', undefined);
        }
      });
    }
  }
  
  public buttons() {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const toolbarSlot = gridDestino.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = false;
      if (e.name === 'saveButton') e.disabled = false;
    });
  }

  public periodoDestinoAte({ component }: IEventParam<ZdDate>) {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const datasource = gridDestino.datasource.currentRow;
    const dataProcessamento = this.utilities.dtProcessFormatada;

    // Encerrar a função se o valor de dataProcessamento for null
    if (!dataProcessamento) {
      this.utilities.showAlert(
        'alert',
        'Não foi encontrada data de processamento, necessário logar novamente no sistema',
        'alert',
      );
      return;
    }

    const periodoDeOrigemInicio = dayjs(datasource.DTINICARDPME, this.utilities.getFormatLocale());
    const periodoDeOrigemFim = dayjs(datasource.DTFINCARDPME, this.utilities.getFormatLocale());
    const periodoDeDestinoInicio = dayjs(component.value, this.utilities.getFormatLocale());

    if (!periodoDeOrigemInicio.isValid() || !periodoDeOrigemFim.isValid() || !periodoDeDestinoInicio.isValid()) {
      gridDestino.updateCell(datasource.PK, 'DTATE_DESTINO', null);
      gridDestino.updateCell(datasource.PK, 'DTDE_DESTINO', null);
      gridDestino.updateCell(datasource.PK, 'DTPERINI_DESTINO', null);
      gridDestino.updateCell(datasource.PK, 'DIASEMANA_DESTINO', null);
      return;
    }

    const diffInDays = periodoDeOrigemFim.diff(periodoDeOrigemInicio, 'day');

    const periodoAteDestino = periodoDeDestinoInicio.add(diffInDays, 'day');

    const dateProcessamento = dayjs(dataProcessamento, this.utilities.getFormatLocale());
    if (!dateProcessamento.isValid()) {
      this.utilities.showAlert('alert', 'Data inicial menor que a data de processamento.', 'error');
      return;
    }

    // Verificar e limpar DTPERINI_DESTINO se estiver fora do intervalo
    const editedRows = gridDestino.getEditedRows(false, true);
    if (editedRows.length > 0) {
      editedRows.forEach((editedRow) => {
        this.buttons();
        if (editedRow.DTPERINI_DESTINO !== null && editedRow.DTPERINI_DESTINO !== undefined) {
          const dtPerIniDestino = dayjs(editedRow.DTPERINI_DESTINO, this.utilities.getFormatLocale());
          const dtDe = dayjs(editedRow.DTDE_DESTINO, this.utilities.getFormatLocale());
          const dtAte = dayjs(editedRow.DTATE_DESTINO, this.utilities.getFormatLocale());

          if (dtPerIniDestino && dtDe && dtAte && (dtPerIniDestino < dtDe || dtPerIniDestino > dtAte)) {
            gridDestino.updateCell(editedRow.PK, 'DTPERINI_DESTINO', null);
            gridDestino.updateCell(editedRow.PK, 'DTDE_DESTINO', null);
            gridDestino.updateCell(editedRow.PK, 'DTPERINI_DESTINO', null);
            gridDestino.updateCell(editedRow.PK, 'DIASEMANA_DESTINO', null);
          }
        }
      });

      const hasInvalidDtPerIni = editedRows.some((editedRow) => {
        if (editedRow.DTPERINI_DESTINO !== null && editedRow.DTPERINI_DESTINO !== undefined) {
          const dtPerIniDestino = dayjs(editedRow.DTPERINI_DESTINO, this.utilities.getFormatLocale());
          return dtPerIniDestino && (dtPerIniDestino < periodoDeDestinoInicio || dtPerIniDestino > periodoAteDestino);
        }
        return false; // Retorno explícito de false para DTPERINI_DESTINO nulo ou undefined
      });

      if (hasInvalidDtPerIni) {
        return;
      }
    }

    if (periodoAteDestino < dateProcessamento) {
      this.utilities.showAlert('alert', 'Data deve ser menor ou igual a data de processamento.', 'warning');
      gridDestino.updateCell(datasource.PK, 'DTATE_DESTINO', null);
      gridDestino.updateCell(datasource.PK, 'DTDE_DESTINO', null);
      component.value = null;
      this.buttons();
    } else {
      this.buttons();
      const formattedInicio = periodoDeDestinoInicio.format(this.utilities.getFormatLocale());
      const formattedFim = periodoAteDestino.format(this.utilities.getFormatLocale());

      gridDestino.updateCell(datasource.PK, 'DTATE_DESTINO', formattedFim);

      if (gridDestino.datasource.data.length > 1) {
        this.askFieldReplicationPer({
          DTDE_DESTINO: formattedInicio,
          DTATE_DESTINO: formattedFim,
        });
      }
    }
  }

  public async saveEditingGridDest() {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const editedRows = gridDestino.getEditedRows(false, true);
    
    let allRowsValid = true;
    
    editedRows.forEach((editedRow: any) => {
      if (!this.isRowFilled(editedRow)) {
        allRowsValid = false;
        // Para a iteração se encontrar uma linha inválida
      }
    });
  
    if (allRowsValid) {
      gridDestino.editing = false;
      const toolbarSlot = gridDestino.toolbarSlot;
      toolbarSlot.forEach((e: IComponentRender) => {
        if (e.name === 'cancelButton') e.disabled = !e.disabled;
        if (e.name === 'saveButton') e.disabled = !e.disabled;
      });
      this.gridEditController.isInlineEditing = gridDestino.editing;
      
      await gridDestino.saveEditedRows();
      const data = [...gridDestino.datasource.data];
      await gridDestino.datasource.updateData([]);
      await gridDestino.datasource.updateData(data);
    } else {
      this.utilities.showAlert('alertError', 'Verificar campos obrigatórios!', 'error');
    }
  }
  
  private isRowFilled(row: any): boolean {
    return !!(
      row.TPCARD_DESTINO
      && row.CDSERVFILI_DESTINO
      && row.DTDE_DESTINO
      && row.DTATE_DESTINO
      && row.DTPERINI_DESTINO
      && row.DIASEMANA_DESTINO
      && row.DSCARDPME_DESTINO
    );
  }
  
  public changeCardDest({ component }: IEventParam<Select>) {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    if (component) {
      gridDestino.updateCell(gridDestino.datasource.currentRow.PK, 'CDSERVFILI_DESTINO', '');
    }
  }
  
  public filtraServicoDestinoFocus({ component }: IEventParam<Select>) {
    component.datasource.data = [];
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const centralDest = formCentralDest.value.CENTRAL_DESTINO;
    
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const editedRows = gridDestino.getEditedRows(false, true);
    if (editedRows.length > 0) {
      editedRows.forEach((row: any) => {
        const filter = [
          {
            name: 'P_LISTATPCARD',
            operator: '=',
            value: row.TPCARD_DESTINO,
          },
          {
            name: 'P_TODOSTPCARD',
            operator: '=',
            value: row.TPCARD_DESTINO !== '' ? null : 'T',
          },
          {
            name: 'P_CDGRUPTIPOCARD',
            operator: '=',
            value: null,
          },
          {
            name: 'P_TGRUPTIPOCARD',
            operator: '=',
            value: 'T',
          },
          {
            name: 'P_CDFILCENTPME',
            operator: '=',
            value: centralDest,
          },
          {
            name: 'P_CDGRUPSERV',
            operator: '=',
            value: null,
          },
          {
            name: 'P_TCDGRUPSERV',
            operator: '=',
            value: 'T',
          },
        ];
        component.datasource.filter = filter;
        component.datasource.get();
      });
    } else {
      const datasource = gridDestino.datasource.currentRow;
      
      const filter = [
        {
          name: 'P_LISTATPCARD',
          operator: '=',
          value: datasource.TPCARD_DESTINO,
        },
        {
          name: 'P_TODOSTPCARD',
          operator: '=',
          value: datasource.TPCARD_DESTINO !== null ? null : 'T',
        },
        {
          name: 'P_CDGRUPTIPOCARD',
          operator: '=',
          value: null,
        },
        {
          name: 'P_TGRUPTIPOCARD',
          operator: '=',
          value: 'T',
        },
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: centralDest,
        },
        {
          name: 'P_CDGRUPSERV',
          operator: '=',
          value: null,
        },
        {
          name: 'P_TCDGRUPSERV',
          operator: '=',
          value: 'T',
        },
      ];
      component.datasource.filter = filter;
      component.datasource.get();
    }
  }
  
  public clearServDest() {
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const editedRows = grid.getEditedRows(false, true);
    
    const currentRow = grid.datasource.currentRow;
    const editedRow = editedRows.find((row) => row.PK === currentRow.PK);
    
    if (editedRow) {
      grid.updateCell(editedRow.PK, 'CDSERVFILI_DESTINO', null);
      grid.updateCell(editedRow.PK, 'CDSERVFILI_DESTINO', undefined);
    }
  }

  public replaceDesc({ component }: IEventParam<TextInput>) {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    if (component.value.length > 0 && component.value.length <= 40) {
      if (gridDestino.datasource.data.length > 1) {
        this.askFieldReplication({
          DSCARDPME_DESTINO: component.value,
        });
      }
      this.buttons();
    }
  }
  
  public setGridDestino() {
    const formCentralDest = Metadata.getInstance<Form>('formCentralDestino');
    const formDadosOrigem = Metadata.getInstance<Form>('formDadosOrigem');
    const grid = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    
    const dataToSend = {
      formCentralDest: formCentralDest.value,
      formDadosOrigem: formDadosOrigem.value,
    };
    
    // if (JSON.stringify(dataToSend) !== JSON.stringify(this.filterGridDestino)) {
    interface ResultItem {
      DTINICARDPME: string;
      DTFINCARDPME: string;
      DTINIPERI: string;
    }
    
    Http.post('/setDataSourceGridDestino', dataToSend).then((response: IDictionary) => {
      const result: ResultItem[] = response.data.dataset.result;
      
      if (result.length > 0) {
        this.filterGridDestino = dataToSend;
        const formattedResult = result.map((item: ResultItem) => ({
          ...item,
          DTINICARDPME: item.DTINICARDPME
            ? dayjs(item.DTINICARDPME, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) : null,
          DTFINCARDPME: item.DTFINCARDPME
            ? dayjs(item.DTFINCARDPME, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) : null,
          DTINIPERI: item.DTINIPERI
            ? dayjs(item.DTINIPERI, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) : null,
          DIASEMANA: item.DTINICARDPME
            ? dayjs(item.DTINICARDPME, 'DD/MM/YYYY').format('ddd') : null,
        }));
        grid.datasource.updateData(formattedResult).then(() => {
          document.getElementsByClassName('gridDadosDestinoIntegracao')[0].scrollIntoView();
        });
      } else {
        grid.datasource.updateData([]);
      }
    });
    // } else {
    //   grid.datasource.updateData(this.lastDataGridDestino).then(() => {
    //     document.getElementsByClassName('gridDadosDestinoIntegracao')[0].scrollIntoView();
    //   });
    // }
  }
  
  public exportar() {
    const gridDestino = Metadata.getInstance<GridEditable>('gridDadosDestinoIntegracao');
    const rows = gridDestino.selectedRows;
    const editedRows = gridDestino.getEditedRows(false, true);
    const formCentralDest: Form = Metadata.getInstance('formCentralDestino');
    const centralDest = formCentralDest.value.CENTRAL_DESTINO;
    const formOrigem: Form = Metadata.getInstance('formDadosOrigem');
    const centralOrigem = formOrigem.value.CENTRAL_ORIGEM;
    
    interface CustoRow {
      DTREFCUSTCAR: string;
      DTPERINI_DESTINO: string; // Ensure this field is included
      VRCUSPADTCAR: number;
      CDSERVFILI: string;
      CDTIPOCARD: string;
    }

    const areAllFieldsFilled = (row: any) => {
      const requiredFields = [
        'TPCARD_DESTINO', 'CDSERVFILI_DESTINO', 'DTDE_DESTINO', 'DTATE_DESTINO',
        'DTPERINI_DESTINO', 'DIASEMANA_DESTINO', 'DSCARDPME_DESTINO',
      ];
      return requiredFields.every((field) => row[field] !== null && row[field] !== undefined);
    };

    const combinedRows = rows.map((selectedRow) => {
      const foundEditedRow = editedRows.find((er) => er.PK === selectedRow.PK);
      const combinedRow = foundEditedRow ? { ...selectedRow, ...foundEditedRow } : selectedRow;
      return { ...combinedRow, CENTRAL_DESTINO: centralDest, CENTRAL_ORIGEM: centralOrigem };
    });

    if (combinedRows.length > 0) {
      if (combinedRows.every(areAllFieldsFilled)) {
        LoadingService.show(' ');
        Http.post('/verificaCardapioPeriodo', { requestType: 'Row', rows: combinedRows }).then((response) => {
          const result = response.data.dataset.result;
          // let desc = '';
          const ate = I18n.translate('até');
          const errorDescriptions: string[] = [];
          if (result.length > 0) {
            result.forEach((error: any) => {
              const matchingRows = combinedRows.filter(
                (combinedRow) => combinedRow.DTINICARDPME === error.DTINICARDPME
                && combinedRow.DTFINCARDPME === error.DTFINCARDPME,
              );
              
              matchingRows.forEach((matchingRow: any) => {
                errorDescriptions.push(`${matchingRow.DTDE_DESTINO} ${ate} ${matchingRow.DTATE_DESTINO}`);
              });
            });
            
            const uniqueErrorDescriptions = Array.from(new Set(errorDescriptions));
            const errorMessage = uniqueErrorDescriptions.join(', ');
            
            this.utilities.showAlert(
              'alertError',
              I18n.translate('O(s) seguinte(s) período(s): {{PARAM1}} está(ão)  '
                + 'compreendido(s) em um período já cadastrado.', { PARAM1: errorMessage }),
              'error',
            );
          } else {
            Http.post('/verificaConfigCardServ', { requestType: 'Row', rows: combinedRows }).then((configPadrao) => {
              const resultConf = configPadrao.data.dataset.result;
              if (resultConf.length > 0) {
                combinedRows.forEach((row) => {
                  if (this.utilities.dateIsValid(row.DTDE_DESTINO, row.DTATE_DESTINO)) {
                    Http.post('/validaCustoPadrao', { requestType: 'Row', rows: combinedRows })
                      .then((validaCustoResponse) => {
                        const custoResult: CustoRow[] = validaCustoResponse.data.dataset.result;

                        const missingCustoRows = combinedRows.filter(
                          (combinedRow) => !custoResult.some(
                            (custoRow) => custoRow.CDSERVFILI === combinedRow.CDSERVFILI_DESTINO
                                        && custoRow.CDTIPOCARD === combinedRow.TPCARD_DESTINO,
                          ),
                        );

                        if (missingCustoRows.length > 0) {
                          const missingDesc = missingCustoRows
                            .map(
                              (item) => `<br/> Tipo de Cardápio: ${item.TPCARD_DESTINO} - 
                              Serviço: ${item.CDSERVFILI_DESTINO}`,
                            );
                          this.utilities.showAlert('alertError',
                            `Os seguintes cardápios e serviços não possuem custo padrão: ${missingDesc}`, 'error', 0);
                        } else {
                          const isValid = custoResult.every((custoRow: CustoRow) => {
                            const dtRefCustoCar = new Date(custoRow.DTREFCUSTCAR.split('/').reverse().join('-'));
                            const matchingCombinedRow = combinedRows.find(
                              (combinedRow) => combinedRow.CDSERVFILI_DESTINO === custoRow.CDSERVFILI
                              && combinedRow.TPCARD_DESTINO === custoRow.CDTIPOCARD,
                            );
                            if (!matchingCombinedRow) {
                              return false;
                            }
                            const dtPerIniDestinoStr = matchingCombinedRow.DTPERINI_DESTINO!;
                            const dtPerIniDestino = new Date(dtPerIniDestinoStr.split('/').reverse().join('-'));
                            return dtRefCustoCar < dtPerIniDestino && custoRow.VRCUSPADTCAR > 0;
                          });

                          if (isValid) {
                            Http.post('/insertListCardAssisIntegra', { requestType: 'Row', rows: combinedRows }).then(() => {
                              this.utilities.showAlert('alertError', 'Exportação realizada com sucesso!', 'green');
                            }).finally(() => {
                              LoadingService.hide();
                            });
                          } else {
                            this.utilities.showAlert('alertError', 'Validação de custo padrão falhou.', 'error');
                            LoadingService.hide();
                          }
                        }
                      });
                  } else {
                    this.utilities.showAlert('invalidDate', 'Data inicial maior que a data final', 'error');
                  }
                });
              } else {
                this.utilities.showAlert('alertError',
                  'Não existe configuração de cardápio cadastrada para o serviço informado',
                  'red');
              }
            });
          }
        }).finally(() => {
          LoadingService.hide();
        });
      } else {
        this.utilities.showAlert('alertError', 'Verificar campos obrigatórios!', 'error');
      }
    } else {
      this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
    }
  }
}
