import {
  Loader, Metadata, IEventParam, IDictionary, Config,
} from '@zeedhi/core';
import {
  Form, Select, Button,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme08000CustoPlanejadoCardController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');

  public modalController = Loader.getInstance('ModalController');

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public setFilterServ({ component }: IEventParam<Select>) {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    let tpCardValue = '';
    if (tipoCard.value.length > 0 && Array.isArray(tipoCard.value)) {
      tipoCard.value.forEach((item: IDictionary) => {
        tpCardValue += `${item.CDTIPOCARD}_`;
      });
    } else tpCardValue = '';

    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tpCardValue,
      },
    ];

    component.datasource.filter = filter;
    component.datasource.get();
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');

    if (this.utilities.centralIsValid() && this.formIsValid(form)) {
      const row: IDictionary = form.value;

      this.setParams(row);
      
      // const dtFinMax = moment(row.DTDE, 'DD/MM/YYYY').add(15, 'd');
      if (row.CDFILCENTPME == null) {
        this.utilities.showAlert('alertError', 'Central não informada.', 'error');
      } else {
        this.reportController.callGenerateReport(component, row, 'report/custoPlanCard');
      }
    }
  }

  private setParams(row: IDictionary) {
    const maskProd: string = (Config.masks.productMask as PmeMask).pattern;
    const maskPrat: string = (Config.masks.plateMask as PmeMask).pattern;
    row.CDTIPOCARD = row.TIPOCARD ? row.TIPOCARD : null;
    row.CDTIPOCUSTO = row.TIPOCUSTO.CDTIPOCUSTO;
    row.NMTIPOCUSTO = row.TIPOCUSTO.NMTIPOCUSTO;
    row.CDNUTRICIO = row.NUTRICIONISTA.CDNUTRICIO;
    row.NMOPERADOR = row.NUTRICIONISTA.NMOPERADOR;
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.CDFILIAL = row.UNIDADE ? row.UNIDADE : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.CDPICTPROD = this.utilities.formatReportMask(maskProd);
    row.CDPICTPRAT = this.utilities.formatReportMask(maskPrat);
  }

  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    if (this.utilities.centralIsValid() && form.validate()) {
      if (this.utilities.dateIsValid(row.DTDE, row.DTATE)) {
        valid = true;
      } else this.utilities.showAlert('alertError', 'Data Inicial deve ser menor que a data final.', 'error');
    }
    return valid;
  }

  public clearFields() {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    unidade.setValue('');
  }
}
