import {
  Loader, Metadata, IEventParam, IDictionary, dayjs as moment,
} from '@zeedhi/core';
import {
  Form, Button, Date as ZdDate,
} from '@zeedhi/common';

export class Pme08200CustoCardapioMesController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');

  public defaultDate({ component }: IEventParam<ZdDate>) {
    this.utilities.setDataProcessamentoZdDate({ component });
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    const row: IDictionary = form.value;
    if (this.utilities.centralIsValid() && form.validate()) {
      this.setParams(row);
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'Central não informada.', 'error');
      else {
        this.reportController.callGenerateReport(component, row, 'report/custoCardMes');
      }
    }
  }
  
  private setParams(row: IDictionary) {
    const format = 'DD/MM/YYYY';
    row.DTINICONSPLAN = moment(row.DATA, format).startOf('month').format('DD/MM/YYYY');
    row.DTINI = moment(row.DTINICONSPLAN, 'DD/MM/YYYY').day(0).format('DD/MM/YYYY');
    row.DTFINCONSPLAN = moment(row.DATA, format).endOf('month').format('DD/MM/YYYY');
    row.DTFIN = moment(row.DTFINCONSPLAN, 'DD/MM/YYYY').day(6).format('DD/MM/YYYY');
    row.CDTIPOCARD = row.TIPOCARD ? row.TIPOCARD.CDTIPOCARD : null;
    row.NMTIPOCARD = row.TIPOCARD ? row.TIPOCARD.NMTIPOCARD : null;
    row.NMFILIAGRE = row.UNIDADE ? row.UNIDADE.NMFILIAL : null;
    row.CDFILIAGRE = row.UNIDADE ? row.UNIDADE.CDFILIAL : null;
    row.CDTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.CDTIPOCUSTO : null;
    row.NMTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.NMTIPOCUSTO : null;
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }
}
